package betterwithmods.common.blocks.mechanical;

import betterwithmods.api.block.IAxle;
import betterwithmods.api.block.IMechanical;
import betterwithmods.api.block.IMechanicalBlock;
import betterwithmods.api.block.IMultiVariants;
import betterwithmods.client.BWCreativeTabs;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWSounds;
import betterwithmods.common.blocks.BlockRotate;
import betterwithmods.util.DirUtils;
import betterwithmods.util.InvUtils;
import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.Random;

import static net.minecraft.util.EnumFacing.Axis.Y;

public class BlockAxle extends BlockRotate implements IMechanical, IAxle, IMultiVariants {
    public static final PropertyEnum<EnumFacing.Axis> AXIS = PropertyEnum.<EnumFacing.Axis>func_177709_a("axis", EnumFacing.Axis.class);
    public static final PropertyInteger SIGNAL = PropertyInteger.func_177719_a("signal", 0, 4);
    public static final int TICK_RATE = 1;

    private static final AxisAlignedBB X_AABB = new AxisAlignedBB(0.0F, 0.375F, 0.375F, 1.0F, 0.625F, 0.625F);
    private static final AxisAlignedBB Y_AABB = new AxisAlignedBB(0.375F, 0.0F, 0.375F, 0.625F, 1.0F, 0.625F);
    private static final AxisAlignedBB Z_AABB = new AxisAlignedBB(0.375F, 0.375F, 0.0F, 0.625F, 0.625F, 1.0F);

    //TODO: Make a reinforced axle with a max power of 5
    public BlockAxle() {
        super(Material.field_151575_d);
        this.func_149711_c(2.0F);
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(AXIS, Y).func_177226_a(SIGNAL, 0));
        this.func_149672_a(SoundType.field_185848_a);
        this.setHarvestLevel("axe", 0);
        func_149647_a(BWCreativeTabs.BWTAB);
    }

    @Override
    public String[] getVariants() {
        return new String[]{"axis=y,signal=0"};
    }

    @Override
    public void nextState(World world, BlockPos pos, IBlockState state) {
        world.func_175656_a(pos, state.func_177226_a(SIGNAL, 0).func_177231_a(AXIS));
    }

    @Override
    public int func_149738_a(World world) {
        return TICK_RATE;
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public int func_180651_a(IBlockState state) {
        return 0;
    }

    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos) {
        switch (state.func_177229_b(AXIS)) {
            case X:
                return X_AABB;
            case Y:
                return Y_AABB;
            case Z:
            default:
                return Z_AABB;
        }
    }

    @Override
    public void func_176213_c(World world, BlockPos pos, IBlockState state) {
        super.func_176213_c(world, pos, state);
        world.func_180497_b(pos, this, TICK_RATE, 5);
    }

    @Override
    public void func_189540_a(IBlockState state, World world, BlockPos pos, Block block, BlockPos other) {
        validatePowerLevel(world, pos);
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random rand) {
        setPowerLevel(world, pos, 0);
        validatePowerLevel(world, pos);
    }

    private void setPowerLevel(World world, BlockPos pos, int power) {
        IBlockState state = world.func_180495_p(pos);
        world.func_175656_a(pos, state.func_177226_a(SIGNAL, power));
    }

    private void validatePowerLevel(World world, BlockPos pos) {
        int currentPower = getPowerLevel(world, pos);
        int maxNeighborPower = 0;
        int greaterPowerNeighbors = 0;

        EnumFacing.Axis axis = world.func_180495_p(pos).func_177229_b(AXIS);
        for (EnumFacing dir : EnumFacing.field_82609_l) {
            if (dir.func_176740_k() == axis) {
                BlockPos pos2 = pos.func_177972_a(dir); //position of block checked
                Block block = world.func_180495_p(pos2).func_177230_c();

                if (block instanceof IMechanical) {
                    IMechanical mech = (IMechanical) block;
                    int powerLevel = mech.getMechPowerLevelToFacing(world, pos2, dir.func_176734_d());

                    if (powerLevel > maxNeighborPower)
                        maxNeighborPower = powerLevel;
                    if (powerLevel > currentPower)
                        greaterPowerNeighbors++;
                }
            }
        }

        if (greaterPowerNeighbors >= 2) {
            breakAxle(world, pos);
            return;
        }

        int newPower;

        if (maxNeighborPower > currentPower) {
            if (maxNeighborPower == 1) {
                breakAxle(world, pos);
                return;
            }
            newPower = maxNeighborPower - 1;
        } else
            newPower = 0;

        if (newPower != currentPower)
            setPowerLevel(world, pos, newPower);
    }

    public void breakAxle(World world, BlockPos pos) {
        if (!world.field_72995_K) {
            InvUtils.ejectStackWithOffset(world, pos, new ItemStack(BWMBlocks.AXLE, 1, 0));
            world.func_175698_g(pos);
        }
    }

    @Override
    public int getMechPowerLevelToFacing(World world, BlockPos pos, EnumFacing dir) {
        EnumFacing.Axis axis = world.func_180495_p(pos).func_177229_b(AXIS);
        if (dir.func_176740_k() == axis)
            return getPowerLevel(world, pos);
        else
            return 0;
    }

    @Override
    public boolean isMechanicalJunction() {
        return false;
    }

    @Override
    public int getPowerLevel(IBlockAccess world, BlockPos pos) {
        return getPowerLevelFromState(world.func_180495_p(pos));
    }

    private int getPowerLevelFromState(IBlockState state) {
        return state.func_177229_b(SIGNAL);
    }

    @Override
    public int getAxisAlignment(IBlockAccess world, BlockPos pos) {
        return DirUtils.getLegacyAxis(world.func_180495_p(pos).func_177229_b(AXIS));
    }

    @Override
    public boolean isAxleOrientedToFacing(IBlockAccess world, BlockPos pos, EnumFacing dir) {
        EnumFacing.Axis axis = world.func_180495_p(pos).func_177229_b(AXIS);
        return axis == dir.func_176740_k();
    }

    public void overpower(World world, BlockPos pos) {
        EnumFacing.Axis axis = world.func_180495_p(pos).func_177229_b(AXIS);
        for (EnumFacing dir : EnumFacing.field_82609_l) {
            if (dir.func_176740_k() == axis) {
                overpowerBlockToSide(world, pos, axis, dir);
            }
        }
    }

    private void overpowerBlockToSide(World world, BlockPos pos, EnumFacing.Axis sourceAxis, EnumFacing dir) {
        BlockPos pos2 = pos.func_177972_a(dir);

        Block block = world.func_180495_p(pos2).func_177230_c();

        if ((block == BWMBlocks.AXLE)) {
            EnumFacing.Axis axis = world.func_180495_p(pos2).func_177229_b(AXIS);

            if (axis == sourceAxis)
                overpowerBlockToSide(world, pos2, sourceAxis, dir);
        } else if (block instanceof IMechanicalBlock) {
            IMechanicalBlock mech = (IMechanicalBlock) block;

            if (mech.canInputPowerToSide(world, pos2, dir.func_176734_d()))
                mech.overpower(world, pos2);
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_180655_c(IBlockState state, World world, BlockPos pos, Random rand) {
        if (state.func_177229_b(SIGNAL) > 0) {
            emitAxleParticles(world, pos, rand);

            if (rand.nextInt(200) == 0)
                world.func_184134_a(pos.func_177958_n() + 0.5D, pos.func_177956_o() + 0.5D, pos.func_177952_p() + 0.5D, BWSounds.WOODCREAK, SoundCategory.BLOCKS, 0.15F, rand.nextFloat() * 0.1F + 0.5F, false);
        }
    }

    private void emitAxleParticles(World world, BlockPos pos, Random rand) {
        int pow = getPowerLevel(world, pos);
        for (int i = 0; i < pow; i++) {
            float flX = pos.func_177958_n() + rand.nextFloat();
            float flY = pos.func_177956_o() + rand.nextFloat() * 0.5F + 0.625F;
            float flZ = pos.func_177952_p() + rand.nextFloat();
            world.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, flX, flY, flZ, 0.0D, 0.0D, 0.0D);
        }
    }

    private int powerLevelFromMeta(int meta) {
        if (meta < 3)
            return 0;
        else if (meta < 6)
            return 1;
        else if (meta < 9)
            return 2;
        else if (meta < 12)
            return 3;
        else if (meta < 15)
            return 4;
        else
            return 0;
    }

    private EnumFacing.Axis orientationFromMeta(int meta) {
        int powerLevel = 3 * powerLevelFromMeta(meta);
        return DirUtils.getAxisFromLegacy(meta - powerLevel);
    }

    @Override
    public IBlockState getStateForPlacement(World world, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer, EnumHand hand) {
        return func_176223_P().func_177226_a(AXIS, facing.func_176740_k());
    }

    @Override
    public int func_176201_c(IBlockState state) {
        return (state.func_177229_b(SIGNAL) * 3) + DirUtils.getLegacyAxis(state.func_177229_b(AXIS));
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        int power = powerLevelFromMeta(meta);
        EnumFacing.Axis ori = orientationFromMeta(meta);
        return this.func_176223_P().func_177226_a(AXIS, ori).func_177226_a(SIGNAL, power);
    }

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, AXIS, SIGNAL);
    }
}
