package betterwithmods.common.blocks.tile;

import betterwithmods.BWMod;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.BlockAnchor;
import betterwithmods.common.blocks.BlockRope;
import betterwithmods.common.blocks.mechanical.BlockMechMachines;
import betterwithmods.common.entity.EntityExtendingRope;
import betterwithmods.module.GlobalConfig;
import betterwithmods.util.InvUtils;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRailBase;
import net.minecraft.block.BlockRailBase.EnumRailDirection;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3i;
import net.minecraft.world.World;
import net.minecraft.world.chunk.storage.AnvilChunkLoader;

import java.util.*;

public class TileEntityPulley extends TileEntityVisibleInventory {

    private EntityExtendingRope rope;
    private NBTTagCompound ropeTag = null;

    public static final boolean isValidPlatform(Block block) {
        return block == BWMBlocks.PLATFORM || block == BWMBlocks.IRON_WALL;
    }

    private boolean isRedstonePowered() {
        return func_145831_w().func_180495_p(field_174879_c).func_177230_c() != null && func_145831_w().func_175640_z(field_174879_c);
    }

    public boolean isMechanicallyPowered() {
        return func_145831_w().func_180495_p(field_174879_c).func_177230_c() != null
                && func_145831_w().func_180495_p(field_174879_c).func_177230_c() instanceof BlockMechMachines
                && ((BlockMechMachines) func_145831_w().func_180495_p(field_174879_c).func_177230_c()).isMechanicalOn(func_145831_w(), field_174879_c);
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newState) {
        return oldState.func_177230_c() != newState.func_177230_c();
    }

    public boolean isRaising() {
        return !isRedstonePowered() && isMechanicallyPowered();
    }

    public boolean isLowering() {
        return !isRedstonePowered() && !isMechanicallyPowered();
    }

    @Override
    public int getInventorySize() {
        return 4;
    }

    @Override
    public SimpleStackHandler createItemStackHandler() {
        return super.createItemStackHandler();
    }

    @Override
    public String getName() {
        return "inv.pulley.name";
    }

    @Override
    public int getMaxVisibleSlots() {
        return 4;
    }

    public boolean isUseableByPlayer(EntityPlayer player) {
        int x = field_174879_c.func_177958_n();
        int y = field_174879_c.func_177956_o();
        int z = field_174879_c.func_177952_p();
        return player.func_70092_e(x + 0.5D, y + 0.5D, z + 0.5D) <= 64.0D;
    }

    @Override
    public void func_73660_a() {
        if (this.func_145831_w().field_72995_K)
            return;
        tryNextOperation();
    }

    private void tryNextOperation() {
        if (!activeOperation() && this.func_145831_w().func_180495_p(this.field_174879_c).func_177230_c() instanceof BlockMechMachines) {
            if (canGoDown(false)) {
                goDown();
            } else if (canGoUp()) {
                goUp();
            }
        }
    }

    private boolean canGoUp() {
        if (isRaising()) {
            if (putRope(false)) {
                BlockPos lowest = BlockRope.getLowestRopeBlock(func_145831_w(), field_174879_c);
                if (!lowest.equals(field_174879_c)) {
                    return true;
                }
            }
        }
        return false;
    }

    private boolean canGoDown(boolean isMoving) {
        if (isLowering()) {
            if (takeRope(false)) {
                BlockPos newPos = BlockRope.getLowestRopeBlock(func_145831_w(), field_174879_c).func_177977_b();
                IBlockState state = func_145831_w().func_180495_p(newPos);
                boolean flag = !isMoving && state.func_177230_c() == BWMBlocks.ANCHOR
                        && ((BlockAnchor) BWMBlocks.ANCHOR).getFacingFromBlockState(state) == EnumFacing.UP;
                if (newPos.func_177956_o() > 0 && (func_145831_w().func_175623_d(newPos) || state.func_177230_c().func_176200_f(func_145831_w(), newPos) || flag)
                        && newPos.func_177984_a().func_177956_o() > 0) {
                    return true;
                }
            }
        }
        return false;
    }

    private void goUp() {
        BlockPos lowest = BlockRope.getLowestRopeBlock(func_145831_w(), field_174879_c);
        IBlockState state = func_145831_w().func_180495_p(lowest.func_177977_b());
        boolean flag = state.func_177230_c() == BWMBlocks.ANCHOR
                && ((BlockAnchor) BWMBlocks.ANCHOR).getFacingFromBlockState(state) == EnumFacing.UP;
        rope = new EntityExtendingRope(func_145831_w(), field_174879_c, lowest, lowest.func_177984_a().func_177956_o());
        if (!flag || movePlatform(lowest.func_177977_b(), true)) {
            func_145831_w().func_184133_a(null, field_174879_c.func_177977_b(), SoundEvents.field_187571_bR, SoundCategory.BLOCKS,
                    0.4F + (func_145831_w().field_73012_v.nextFloat() * 0.1F), 1.0F);
            func_145831_w().func_72838_d(rope);
            func_145831_w().func_175698_g(lowest);
            putRope(true);
        } else {
            rope = null;
        }
    }

    private void goDown() {
        BlockPos newPos = BlockRope.getLowestRopeBlock(func_145831_w(), field_174879_c).func_177977_b();
        IBlockState state = func_145831_w().func_180495_p(newPos);
        boolean flag = state.func_177230_c() == BWMBlocks.ANCHOR
                && ((BlockAnchor) BWMBlocks.ANCHOR).getFacingFromBlockState(state) == EnumFacing.UP;
        rope = new EntityExtendingRope(func_145831_w(), field_174879_c, newPos.func_177984_a(), newPos.func_177956_o());
        if (!flag || movePlatform(newPos, false)) {
            func_145831_w().func_72838_d(rope);
        } else {
            rope = null;
        }
    }

    /**
     * Turns the platform into entities and moves them with the rope
     */

    private boolean movePlatform(BlockPos anchor, boolean up) {
        IBlockState state = func_145831_w().func_180495_p(anchor);
        if (state.func_177230_c() != BWMBlocks.ANCHOR)
            return false;

        HashSet<BlockPos> platformBlocks = new HashSet<>();
        platformBlocks.add(anchor);
        Block b = func_145831_w().func_180495_p(anchor.func_177977_b()).func_177230_c();
        boolean success = isValidPlatform(func_145831_w().func_180495_p(anchor.func_177977_b()).func_177230_c())
                ? addToList(platformBlocks, anchor.func_177977_b(), up) : up || isValidBlock(b, anchor.func_177977_b());
        if (!success) {
            return false;
        }

        for (BlockPos blockPos : platformBlocks) {
            Arrays.asList(new BlockPos[]{blockPos.func_177978_c(), blockPos.func_177968_d()}).forEach(p -> {
                if (!platformBlocks.contains(p)) {
                    fixRail(p, EnumRailDirection.ASCENDING_NORTH, EnumRailDirection.ASCENDING_SOUTH);
                }
            });
            Arrays.asList(new BlockPos[]{blockPos.func_177974_f(), blockPos.func_177976_e()}).forEach(p -> {
                if (!platformBlocks.contains(p)) {
                    fixRail(p, EnumRailDirection.ASCENDING_EAST, EnumRailDirection.ASCENDING_WEST);
                }
            });
        }

        if (!func_145831_w().field_72995_K) {
            for (BlockPos blockPos : platformBlocks) {
                IBlockState blockState = func_145831_w().func_180495_p(blockPos.func_177984_a());
                b = blockState.func_177230_c();
                blockState = (b == Blocks.field_150488_af || b instanceof BlockRailBase ? blockState : null);
                Vec3i offset = blockPos.func_177973_b(anchor.func_177984_a());
                rope.addBlock(offset, func_145831_w().func_180495_p(blockPos));
                if (blockState != null) {
                    rope.addBlock(new Vec3i(offset.func_177958_n(), offset.func_177956_o() + 1, offset.func_177952_p()), blockState);
                    func_145831_w().func_175698_g(blockPos.func_177984_a());
                }
                func_145831_w().func_175698_g(blockPos);
            }
        }

        return true;
    }

    public boolean isValidBlock(Block b, BlockPos pos) {
        return b == Blocks.field_150350_a || b.func_176200_f(func_145831_w(), pos) || isValidPlatform(b);
    }

    @SuppressWarnings("unchecked")
    private void fixRail(BlockPos rail, EnumRailDirection... directions) {
        List<EnumRailDirection> list = Arrays.asList(directions);
        IBlockState state = func_145831_w().func_180495_p(rail);
        if (func_145831_w().func_180495_p(rail).func_177230_c() instanceof BlockRailBase) {
            PropertyEnum<EnumRailDirection> shape = null;
            for (IProperty<?> p : state.func_177227_a()) {
                if ("shape".equals(p.func_177701_a()) && p instanceof PropertyEnum<?>) {
                    shape = (PropertyEnum<EnumRailDirection>) p;
                    break;
                }
            }

            if (shape != null) {
                EnumRailDirection currentShape = state.func_177229_b(shape);
                if (list.contains(currentShape)) {
                    func_145831_w().func_180501_a(rail, state.func_177226_a(shape, flatten(currentShape)), 6);
                }
            } else {
                Formatter f = new Formatter();
                BWMod.logger.warn(f.format("Rail at %s has no shape?", rail));
                f.close();
            }
        }
    }

    private EnumRailDirection flatten(EnumRailDirection old) {
        switch (old) {
            case ASCENDING_EAST:
            case ASCENDING_WEST:
                return EnumRailDirection.EAST_WEST;
            case ASCENDING_NORTH:
            case ASCENDING_SOUTH:
                return EnumRailDirection.NORTH_SOUTH;
            default:
                return old;
        }
    }

    private boolean addToList(HashSet<BlockPos> set, BlockPos p, boolean up) {
        if (set.size() > GlobalConfig.maxPlatformBlocks)
            return false;

        BlockPos blockCheck = up ? p.func_177984_a() : p.func_177977_b();

        if (!isValidPlatform(func_145831_w().func_180495_p(p).func_177230_c())) {
            return true;
        }

        Block b = func_145831_w().func_180495_p(blockCheck).func_177230_c();

        if (b != Blocks.field_150488_af && !(b instanceof BlockRailBase)) {
            if (!(func_145831_w().func_175623_d(blockCheck) || b.func_176200_f(func_145831_w(), blockCheck) || isValidPlatform(b))
                    && !set.contains(blockCheck)) {
                return false;
            }
        }

        set.add(p);

        List<BlockPos> fails = new ArrayList<>();

        Arrays.asList(p.func_177984_a(), p.func_177977_b(), p.func_177978_c(), p.func_177968_d(), p.func_177974_f(), p.func_177976_e()).forEach(q -> {
            if (fails.isEmpty() && !set.contains(q)) {
                if (!addToList(set, q, up))
                    fails.add(q);
            }
        });

        return fails.isEmpty();
    }

    private boolean activeOperation() {
        return rope != null && rope.func_70089_S();
    }

    private boolean takeRope(boolean flag) {
        return InvUtils.consumeItemsInInventory(inventory, new ItemStack(BWMBlocks.ROPE), 1, !flag);
    }

    private boolean putRope(boolean flag) {
        return InvUtils.insert(inventory, new ItemStack(BWMBlocks.ROPE, 1), !flag).func_190926_b();
    }

    public boolean onJobCompleted(boolean up, int targetY, EntityExtendingRope theRope) {
        BlockPos ropePos = new BlockPos(field_174879_c.func_177958_n(), targetY - (up ? 1 : 0), field_174879_c.func_177952_p());
        IBlockState state = func_145831_w().func_180495_p(ropePos);
        if (!up) {
            if ((func_145831_w().func_175623_d(ropePos) || state.func_177230_c().func_176200_f(func_145831_w(), ropePos)) && BWMBlocks.ROPE.func_176196_c(func_145831_w(), ropePos) && takeRope(true)) {
                func_145831_w().func_184133_a(null, field_174879_c.func_177977_b(), SoundEvents.field_187577_bU, SoundCategory.BLOCKS, 0.4F, 1.0F);
                func_145831_w().func_175656_a(ropePos, BWMBlocks.ROPE.func_176223_P());
            } else {
                tryNextOperation();
                theRope.func_70106_y();
                return false;
            }
        }
        if ((theRope.getUp() ? canGoUp() : canGoDown(true)) && !theRope.isPathBlocked()) {
            theRope.setTargetY(targetY + (theRope.getUp() ? 1 : -1));
            if (up) {
                if (!func_145831_w().func_175623_d(ropePos.func_177984_a())) {
                    func_145831_w().func_184133_a(null, field_174879_c.func_177977_b(), SoundEvents.field_187571_bR, SoundCategory.BLOCKS,
                            0.4F + (func_145831_w().field_73012_v.nextFloat() * 0.1F), 1.0F);
                    func_145831_w().func_175698_g(ropePos.func_177984_a());
                    putRope(true);
                }
            }
            return true;
        } else {
            tryNextOperation();
            theRope.func_70106_y();
            return false;
        }
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        NBTTagCompound ropetag = new NBTTagCompound();
        if (rope != null)
            rope.func_184198_c(ropetag);
        tag.func_74782_a("Rope", ropetag);
        return super.func_189515_b(tag);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);
        this.ropeTag = (NBTTagCompound) tag.func_74781_a("Rope");
    }

    @Override
    public void func_145834_a(World worldIn) {
        super.func_145834_a(worldIn);
        if (rope == null && !worldIn.field_72995_K && ropeTag != null && !ropeTag.func_82582_d()) {
            NBTTagList pos = (NBTTagList) ropeTag.func_74781_a("Pos");
            if (pos != null) {
                rope = (EntityExtendingRope) AnvilChunkLoader.func_186054_a(ropeTag, func_145831_w(), pos.func_150309_d(0),
                        pos.func_150309_d(1), pos.func_150309_d(2), true);
            }
        }
    }

//    private class PulleyInventory extends SimpleStackHandler {
//
//        public PulleyInventory(int size, TileEntity tile) {
//            super(size, tile);
//        }
//
//        @Override
//        @Nonnull
//        public ItemStack insertItem(int slot, @Nonnull ItemStack stack, boolean simulate) {
////            if (!stack.isEmpty() && stack.getItem() instanceof ItemBlock && stack.getItem() == Item.getItemFromBlock(BWMBlocks.ROPE)) {
////
////            }
////            return ItemStack.EMPTY;
//            return super.insertItem(slot, stack, simulate);
//        }
//    }

}
