package betterwithmods.common.blocks.tile.gen;

import betterwithmods.api.block.IAxle;
import betterwithmods.api.block.IMechanicalBlock;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWMItems;
import betterwithmods.common.blocks.mechanical.BlockAxle;
import betterwithmods.common.blocks.mechanical.BlockWindmill;
import betterwithmods.util.InvUtils;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.DimensionType;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class TileEntityWindmillVertical extends TileEntityMillGenerator implements IColor {
    public int[] bladeMeta = {0, 0, 0, 0, 0, 0, 0, 0};

    public int getBladeColor(int blade) {
        return bladeMeta[blade];
    }

    @Override
    public int getMinimumInput(EnumFacing facing) {
        return 0;
    }

    @Override
    public int getColorFromBlade(int blade) {
        return bladeMeta[blade];
    }

    @Override
    public boolean dyeBlade(int dyeColor) {
        boolean dyed = false;
        if (bladeMeta[dyeIndex] != dyeColor) {
            bladeMeta[dyeIndex] = dyeColor;
            dyed = true;
            IBlockState state = func_145831_w().func_180495_p(this.field_174879_c);
            this.func_145831_w().func_184138_a(this.field_174879_c, state, state, 3);
            this.func_70296_d();
        }
        dyeIndex++;
        if (dyeIndex > 7)
            dyeIndex = 0;
        return dyed;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);
        for (int i = 0; i < 8; i++) {
            if (tag.func_74764_b("Color_" + i))
                bladeMeta[i] = tag.func_74762_e("Color_" + i);
        }
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        NBTTagCompound t = super.func_189515_b(tag);
        for (int i = 0; i < 8; i++) {
            t.func_74768_a("Color_" + i, bladeMeta[i]);
        }
        t.func_74774_a("DyeIndex", this.dyeIndex);
        return t;
    }

    @Override
    public boolean isValid() {
        //check master's validity
        boolean valid = true;
        if (func_145831_w().func_180495_p(field_174879_c).func_177230_c() != null && func_145831_w().func_180495_p(field_174879_c).func_177230_c() == BWMBlocks.WINDMILL_BLOCK) {
            for (int i = -3; i < 4; i++) {
                if (i == 0)
                    continue;
                valid = isSlaveValid(i);
                if (!valid)
                    break;
            }
        }
        return valid;
    }

    public boolean isSlaveValid(int offset) {
        boolean notBlocked = true;
        int airCounter = 0;
        for (int x = -4; x < 5; x++) {
            for (int z = -4; z < 5; z++) {
                BlockPos offPos = field_174879_c.func_177982_a(x, offset, z);
                if (x == 0 && z == 0) {
                    if (func_145831_w().func_180495_p(offPos).func_177230_c() instanceof IAxle && ((IAxle) func_145831_w().func_180495_p(offPos).func_177230_c()).getAxisAlignment(func_145831_w(), offPos) == 0)
                        continue;
                }
                if (func_145831_w().field_73011_w.func_186058_p() == DimensionType.NETHER)
                    notBlocked = this.func_145831_w().func_175623_d(offPos);
                else if (func_145831_w().field_73011_w.func_186058_p() != DimensionType.NETHER) {
                    notBlocked = this.func_145831_w().func_175623_d(offPos);
                    if (func_145831_w().func_175710_j(offPos))
                        airCounter++;
                }
                if (!notBlocked)
                    break;
            }
            if (!notBlocked)
                break;
        }
        if (func_145831_w().field_73011_w.func_186058_p() != DimensionType.NETHER)
            return notBlocked && airCounter > 25;
        return notBlocked;
    }

    @Override
    public boolean verifyIntegrity() {
        //check master's integrity
        boolean integrity = true;
        {
            for (int offY = -3; offY < 4; offY++) {
                BlockPos offset = field_174879_c.func_177982_a(0, offY, 0);
                if (offY == 0)
                    continue;
                integrity = func_145831_w().func_180495_p(offset).func_177230_c() instanceof IAxle && ((IAxle) func_145831_w().func_180495_p(offset).func_177230_c()).getAxisAlignment(func_145831_w(), offset) == 0;
                if (!integrity) {
                    invalidateWindmill();
                    break;
                }
            }
        }
        return integrity;
    }

    private void invalidateWindmill() {
        this.func_145831_w().func_175656_a(this.field_174879_c, BWMBlocks.WINDMILL_BLOCK.func_176223_P());
        for (int i = -3; i < 4; i++) {
            BlockPos pos = this.field_174879_c.func_177982_a(0, i, 0);
            if (func_145831_w().func_180495_p(pos).func_177230_c() instanceof BlockAxle)
                this.func_145831_w().func_175656_a(pos, BWMBlocks.AXLE.func_176223_P());
        }
        if (!this.func_145831_w().field_72995_K)
            InvUtils.ejectStackWithOffset(func_145831_w(), field_174879_c, new ItemStack(BWMItems.WINDMILL, 1, 2));
        this.func_145831_w().func_175656_a(this.field_174879_c, BWMBlocks.AXLE.func_176223_P());
    }

    @Override
    public void overpower() {
        if (this.func_145838_q() instanceof BlockWindmill) {
            EnumFacing.Axis axis = func_145831_w().func_180495_p(field_174879_c).func_177229_b(BlockWindmill.AXIS);
            for (EnumFacing dir : EnumFacing.field_82609_l) {
                if (dir.func_176740_k() == axis) {
                    BlockPos offset = field_174879_c.func_177972_a(dir);
                    Block axle = this.func_145831_w().func_180495_p(offset).func_177230_c();
                    if (axle instanceof BlockAxle)
                        ((BlockAxle) axle).overpower(this.func_145831_w(), offset);
                    else if (axle instanceof IMechanicalBlock && ((IMechanicalBlock) axle).canInputPowerToSide(func_145831_w(), offset, dir.func_176734_d()))
                        ((IMechanicalBlock) axle).overpower(this.func_145831_w(), offset);
                }
            }
        }
    }



    @Override
    @SideOnly(Side.CLIENT)
    public void onDataPacket(NetworkManager mgr, SPacketUpdateTileEntity pkt) {
        NBTTagCompound tag = pkt.func_148857_g();
        this.func_145839_a(tag);
        IBlockState state = func_145831_w().func_180495_p(this.field_174879_c);
        this.func_145831_w().func_184138_a(this.field_174879_c, state, state, 3);
    }

    @Override
    public NBTTagCompound func_189517_E_() {
        return func_189515_b(new NBTTagCompound());
    }

    @Override
    public void updateSpeed() {
        byte speed = 0;
        if (this.isValid() && !isGalacticraftDimension() && isNotOtherDimension()) {
            if ((this.func_145831_w().func_72896_J() || this.func_145831_w().func_72911_I()) && this.func_145831_w().field_73011_w.func_186058_p() != DimensionType.NETHER)
                speed = 2;
            else
                speed = 1;
        }
        if (speed != this.runningState && func_145831_w().func_180495_p(field_174879_c).func_177230_c() instanceof BlockWindmill) {
            this.setRunningState(speed);
            this.func_145831_w().func_175656_a(field_174879_c, this.func_145831_w().func_180495_p(field_174879_c).func_177226_a(BlockWindmill.ISACTIVE, speed > 0));
            func_145831_w().func_180497_b(field_174879_c, this.func_145838_q(), this.func_145838_q().func_149738_a(func_145831_w()), 5);//this.getWorld().markBlockForUpdate(pos);
        }
    }

    public boolean isNotOtherDimension() {
        return this.func_145831_w().field_73011_w.func_186058_p() != DimensionType.THE_END;
    }

    public boolean isGalacticraftDimension() {
        boolean isDimension = false;
        if (Loader.isModLoaded("GalacticraftCore")) {
            isDimension = false;//GalacticraftCompat.isGalacticraftDimension(this.getWorld());
        }
        return isDimension;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox() {
        int x = field_174879_c.func_177958_n();
        int y = field_174879_c.func_177956_o();
        int z = field_174879_c.func_177952_p();
        if (func_145831_w().func_180495_p(field_174879_c).func_177230_c() != null && func_145831_w().func_180495_p(field_174879_c).func_177230_c() instanceof BlockWindmill)
            return new AxisAlignedBB(x - 4, y - 4, z - 4, x + 4, y + 4, z + 4);
        else
            return super.getRenderBoundingBox();
    }
}
